# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui

from prettyqt.utils import bidict


MOVE_MODES = bidict(move=QtGui.QTextCursor.MoveAnchor,
                    keep=QtGui.QTextCursor.KeepAnchor)

MOVE_OPERATIONS = bidict(no_move=QtGui.QTextCursor.NoMove,
                         start=QtGui.QTextCursor.Start,
                         start_of_line=QtGui.QTextCursor.StartOfLine,
                         start_of_block=QtGui.QTextCursor.StartOfBlock,
                         start_of_word=QtGui.QTextCursor.StartOfWord,
                         previous_block=QtGui.QTextCursor.PreviousBlock,
                         previous_char=QtGui.QTextCursor.PreviousCharacter,
                         previous_word=QtGui.QTextCursor.PreviousWord,
                         up=QtGui.QTextCursor.Up,
                         left=QtGui.QTextCursor.Left,
                         word_left=QtGui.QTextCursor.WordLeft,
                         end=QtGui.QTextCursor.End,
                         end_of_line=QtGui.QTextCursor.EndOfLine,
                         end_of_word=QtGui.QTextCursor.EndOfWord,
                         end_of_block=QtGui.QTextCursor.EndOfBlock,
                         next_block=QtGui.QTextCursor.NextBlock,
                         next_char=QtGui.QTextCursor.NextCharacter,
                         next_word=QtGui.QTextCursor.NextWord,
                         down=QtGui.QTextCursor.Down,
                         right=QtGui.QTextCursor.Right,
                         word_right=QtGui.QTextCursor.WordRight,
                         next_cell=QtGui.QTextCursor.NextCell,
                         previous_cell=QtGui.QTextCursor.PreviousCell,
                         next_row=QtGui.QTextCursor.NextRow,
                         previous_row=QtGui.QTextCursor.PreviousRow)

SELECTION_TYPES = bidict(document=QtGui.QTextCursor.Document,
                         block_under_cursor=QtGui.QTextCursor.BlockUnderCursor,
                         line_under_cursor=QtGui.QTextCursor.LineUnderCursor,
                         word_under_cursor=QtGui.QTextCursor.WordUnderCursor)


class TextCursor(QtGui.QTextCursor):

    def move_position(self, operation: str, mode: str = "move", n: int = 1) -> bool:
        op = MOVE_OPERATIONS[operation]
        mode = MOVE_MODES[mode]
        return self.movePosition(op, mode, n)

    def set_position(self, pos: int, mode: str = "move"):
        self.setPosition(pos, MOVE_MODES[mode])

    def select(self, selection):
        if selection in SELECTION_TYPES:
            selection = SELECTION_TYPES[selection]
        super().select(selection)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from typing import Any, Callable, Optional, Union

import qtawesome as qta
from qtpy import QtGui, QtWidgets

from prettyqt import core, gui, widgets

QtWidgets.QMenu.__bases__ = (widgets.Widget,)


class Menu(QtWidgets.QMenu):

    def __init__(self, title="", icon=None, parent=None):
        super().__init__(title, parent=parent)
        self.set_icon(icon)
        self.setToolTipsVisible(True)

    def __iter__(self):
        return iter(self.actions())

    def __len__(self):
        return len(self.actions())

    def __add__(self, other):
        if isinstance(other, QtWidgets.QAction):
            self.add(other)
            return self
        raise TypeError("Invalid Type")

    def __getitem__(self, item):
        for action in self.actions():
            if action.id == item:
                return action
        raise KeyError(f"Action {item} not in menu")

    def add(self, *item):
        for i in item:
            self.add_action(i)

    def set_icon(self, icon: Union[QtGui.QIcon, str, None]):
        """set the icon for the menu

        Args:
            icon: icon to use
        """
        if not icon:
            icon = gui.Icon()
        elif isinstance(icon, str):
            icon = qta.icon(icon)
        self.setIcon(icon)

    def add_separator(self, text: Optional[str] = None) -> widgets.WidgetAction:
        """adds a separator showing an optional label

        Args:
            text: Text to show on separator

        Returns:
            Separator action
        """
        separator = widgets.WidgetAction(parent=self)
        if text is None:
            separator.setSeparator(True)
        else:
            label = widgets.Label(text)
            label.setMinimumWidth(self.minimumWidth())
            label.setStyleSheet("background:lightgrey")
            label.set_alignment(horizontal="center")
            separator = widgets.WidgetAction(parent=self)
            separator.setDefaultWidget(label)
        self.add(separator)
        return separator

    def add_action(self,
                   label: Union[str, widgets.Action],
                   callback: Callable = None,
                   icon: Optional[Any] = None,
                   checkable: bool = False,
                   checked: bool = False,
                   shortcut: Optional[str] = None,
                   status_tip: Optional[str] = None
                   ) -> widgets.Action:
        """Add an action to the menu

        Args:
            label: Label for button
            callback: gets called when action is triggered
            icon: icon for button (default: {None})
            checkable: as checkbox button (default: {False})
            checked: if checkable, turn on by default (default: {False})
            shortcut: Shortcut for action (a) (default: {None})
            status_tip: Status tip to be shown in status bar (default: {None})

        Returns:
            Action added to menu
        """
        if isinstance(label, str):
            action = widgets.Action(text=label, parent=self)
            if callback:
                action.triggered.connect(callback)
            action.set_icon(icon)
            action.set_shortcut(shortcut)
            if checkable:
                action.setCheckable(True)
                action.setChecked(checked)
            if status_tip is not None:
                action.setStatusTip(status_tip)
        else:
            action = label
            action.setParent(self)
        self.addAction(action)
        return action

    def add_actions(self, actions):
        self.addActions(actions)

    def add_menu(self, menu: QtWidgets.QMenu):
        return self.addAction(menu.menuAction())


if __name__ == "__main__":
    app = widgets.app()
    menu = Menu("1")
    action = widgets.Action(text="test")
    menu.addAction(action)
    menu.show()
    menu.exec_(core.Point(200, 200))



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
based on http://www.yasinuludag.com/blog/?p=49
"""

from prettyqt import core, gui


class XmlHighlighter(gui.SyntaxHighlighter):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.highlight_rules = []

        xmlElementFormat = gui.TextCharFormat()
        xmlElementFormat.set_foreground_color("blue")
        self.highlight_rules.append((core.RegExp(r"\\b[A-Za-z0-9_]+(?=[\s/>])"),
                                     xmlElementFormat))
        xml_attr_format = gui.TextCharFormat()
        xml_attr_format.setFontItalic(True)
        xml_attr_format.set_foreground_color("lightgreen")
        self.highlight_rules.append((core.RegExp(r"\\b[A-Za-z0-9_]+(?=\\=)"),
                                     xml_attr_format))
        self.highlight_rules.append((core.RegExp(r"="), xml_attr_format))

        self.valueFormat = gui.TextCharFormat()
        self.valueFormat.set_foreground_color("orange")
        self.valueStartExpression = core.RegExp(r"\"")
        self.valueEndExpression = core.RegExp(r"\"(?=[\s></])")

        singleLineCommentFormat = gui.TextCharFormat()
        singleLineCommentFormat.set_foreground_color("lightgrey")
        self.highlight_rules.append((core.RegExp(r"<!--[^\n]*-->"),
                                     singleLineCommentFormat))

        textFormat = gui.TextCharFormat()
        textFormat.set_foreground_color("black")
        # (?<=...)  - lookbehind is not supported
        self.highlight_rules.append((core.RegExp(r">(.+)(?=</)"), textFormat))

        keywordFormat = gui.TextCharFormat()
        keywordFormat.set_foreground_color("red")
        keywordFormat.setFontWeight(gui.Font.Bold)
        keywordPatterns = ["\\b?xml\\b", "/>", ">", "<", "</"]
        self.highlight_rules += [(core.RegExp(pattern), keywordFormat)
                                 for pattern in keywordPatterns]

    # VIRTUAL FUNCTION WE OVERRIDE THAT DOES ALL THE COLLORING

    def highlightBlock(self, text):
        # for every pattern
        for pattern, format in self.highlight_rules:
            # Create a regular expression from the retrieved pattern
            expression = core.RegExp(pattern)
            # Check what index that expression occurs at with the ENTIRE text
            index = expression.indexIn(text)
            # While the index is greater than 0
            while index >= 0:
                # Get the length of how long the expression is true,
                # set the format from the start to the length with the text format
                length = expression.matchedLength()
                self.setFormat(index, length, format)
                # Set index to where the expression ends in the text
                index = expression.indexIn(text, index + length)

        # HANDLE QUOTATION MARKS NOW.. WE WANT TO START WITH " AND END WITH "..
        # A THIRD " SHOULD NOT CAUSE THE WORDS INBETWEEN SECOND AND THIRD
        # TO BE COLORED
        self.setCurrentBlockState(0)
        startIndex = 0
        if self.previousBlockState() != 1:
            startIndex = self.valueStartExpression.indexIn(text)
        while startIndex >= 0:
            endIndex = self.valueEndExpression.indexIn(text, startIndex)
            if endIndex == -1:
                self.setCurrentBlockState(1)
                commentLength = len(text) - startIndex
            else:
                matched_len = self.valueEndExpression.matchedLength()
                commentLength = endIndex - startIndex + matched_len
            self.setFormat(startIndex, commentLength, self.valueFormat)
            startIndex = self.valueStartExpression.indexIn(text,
                                                           startIndex + commentLength)


if __name__ == "__main__":
    from prettyqt import widgets
    app = widgets.app()
    editor = widgets.PlainTextEdit()
    highlighter = XmlHighlighter(editor.document())
    editor.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import gui


class MatchHighlighter(gui.SyntaxHighlighter):

    def __init__(self, document):
        super().__init__(document)
        self.prog = None
        self._format = gui.TextCharFormat()
        self._format.set_background_color("lightgreen")
        self.matches = []

    def set_prog(self, prog):
        self.prog = prog
        text = self.document().toPlainText()
        if self.prog is not None:
            self.matches = [m.span() for m in self.prog.finditer(text)]
            # print(self.matches)
        self.rehighlight()

    def highlightBlock(self, text):
        block = self.currentBlock()
        # line_no = block.blockNumber()
        # if line_no == 0:
        #     self.setCurrentBlockState(-1)
        start_char = block.position()
        end_char = start_char + block.length()
        # print(f"\nline {line_no} ({start_char} - {end_char})")
        # print(f"prev block state: {self.previousBlockState()}")
        if not self.matches or not text:
            return None
        for start, end in self.matches:
            match_len = end - start
            starts_in_line = start_char <= start <= end_char
            ends_in_line = start_char <= end <= end_char
            line_pos = start - start_char
            if starts_in_line and ends_in_line:
                # print(f"in line: {line_pos} - {line_pos + match_len}")
                self.setFormat(line_pos, line_pos + match_len, self._format)
            elif ends_in_line:
                # if self.previousBlockState() == 1:
                # print(f"ends: {end}")
                self.setFormat(0, end, self._format)
                self.setCurrentBlockState(-1)
            elif starts_in_line:
                # print(f"starts: {line_pos}")
                self.setCurrentBlockState(1)
                self.setFormat(line_pos, end_char, self._format)
